@@ -27,6 +27,8 @@
 // the License.
 
 use core::ops::{Mul, Sub};
+#[cfg(feature = "no-panic")]
+use no_panic::no_panic;
 
 #[derive(Copy, Clone, Debug)]
 pub struct DiyFp<F, E> {
@@ -35,6 +37,7 @@ pub struct DiyFp<F, E> {
 }
 
 impl<F, E> DiyFp<F, E> {
+    #[cfg_attr(feature = "no-panic", no_panic)]
     pub fn new(f: F, e: E) -> Self {
         DiyFp { f, e }
     }
@@ -45,6 +48,8 @@ where
     F: Sub<F, Output = F>,
 {
     type Output = Self;
+
+    #[cfg_attr(feature = "no-panic", no_panic)]
     fn sub(self, rhs: Self) -> Self {
         DiyFp {
             f: self.f - rhs.f,
@@ -55,6 +60,8 @@ where
 
 impl Mul for DiyFp<u32, i32> {
     type Output = Self;
+
+    #[cfg_attr(feature = "no-panic", no_panic)]
     fn mul(self, rhs: Self) -> Self {
         let mut tmp = self.f as u64 * rhs.f as u64;
         tmp += 1u64 << 31; // mult_round
@@ -67,6 +74,8 @@ impl Mul for DiyFp<u32, i32> {
 
 impl Mul for DiyFp<u64, isize> {
     type Output = Self;
+
+    #[cfg_attr(feature = "no-panic", no_panic)]
     fn mul(self, rhs: Self) -> Self {
         let m32 = 0xFFFFFFFFu64;
         let a = self.f >> 32;
@@ -127,8 +136,9 @@ macro_rules! diyfp {
                 }
             }
             */
+            #[cfg_attr(feature = "no-panic", no_panic)]
             unsafe fn from(d: $fty) -> Self {
-                let u: $mask_type = mem::transmute(d);
+                let u: $mask_type = <$fty>::to_bits(d);
 
                 let biased_e = ((u & $exponent_mask) >> $significand_size) as $expty;
                 let significand = u & $significand_mask;
@@ -156,6 +166,7 @@ macro_rules! diyfp {
                 return res;
             }
             */
+            #[cfg_attr(feature = "no-panic", no_panic)]
             fn normalize(self) -> DiyFp {
                 let mut res = self;
                 while (res.f & (1 << ($diy_significand_size - 1))) == 0 {
@@ -181,6 +192,7 @@ macro_rules! diyfp {
                 return res;
             }
             */
+            #[cfg_attr(feature = "no-panic", no_panic)]
             fn normalize_boundary(self) -> DiyFp {
                 let mut res = self;
                 while (res.f & $hidden_bit << 1) == 0 {
@@ -210,6 +222,7 @@ macro_rules! diyfp {
                 *minus = mi;
             }
             */
+            #[cfg_attr(feature = "no-panic", no_panic)]
             fn normalized_boundaries(self) -> (DiyFp, DiyFp) {
                 let pl = DiyFp::new((self.f << 1) + 1, self.e - 1).normalize_boundary();
                 let mut mi = if self.f == $hidden_bit {
@@ -238,6 +251,7 @@ macro_rules! diyfp {
         }
         */
         #[inline]
+        #[cfg_attr(feature = "no-panic", no_panic)]
         fn get_cached_power(e: $expty) -> (DiyFp, isize) {
             let dk = (3 - $diy_significand_size - e) as f64 * 0.30102999566398114f64
                 - ($min_power + 1) as f64;
@@ -250,7 +264,10 @@ macro_rules! diyfp {
             let k = -($min_power + (index << 3) as isize);
 
             (
-                DiyFp::new($cached_powers_f[index], $cached_powers_e[index] as $expty),
+                DiyFp::new(*unsafe { $cached_powers_f.get_unchecked(index) }, *unsafe {
+                    $cached_powers_e.get_unchecked(index)
+                }
+                    as $expty),
                 k,
             )
         }
